unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, VCLTee.TeEngine,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.Series, VCLTee.TeCanvas;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    Series1: TBarSeries;
    Image1: TImage;
    procedure Chart1Resize(Sender: TObject);
  private
    { Private declarations }
  protected
     procedure Resize; override;
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  {$ifopt C+}
  lCanvasClassName: string;
  {$endif}
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
  lHoldHover: Boolean;
begin
  {$ifopt C+}
  lCanvasClassName := aChart.Canvas.ClassName;
  {$endif}
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);
  if lCanvasClass <> TTeeCanvas3D then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;
  lHoldHover := aChart.Hover.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := VCL.Graphics.clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;
    aChart.Hover.Visible := false;

    lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
    try
      aGraphic.Assign(lBmp);
    finally
      lBmp.Free;
    end;
  finally
    aChart.Hover.Visible := lHoldHover;
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;

  {$ifopt C+}
  assert(SameText(lCanvasClassName, aChart.Canvas.ClassName),
    Format('Chart canvas should be %s but is %s', [lCanvasClassName, aChart.Canvas.ClassName]));
  {$endif}
end;


procedure TForm1.Chart1Resize(Sender: TObject);
var
  lChart: TChart;
  lOneLineTitle: String;
  lTitleHeight: Integer;
  lTitle: TChartAxisTitle;
begin
  Assert(Sender is TChart);
  lChart := TChart(Sender);

  lTitle := lChart.Axes.Left.Title;
  lOneLineTitle := StringReplace(lTitle.Text, sLineBreak, ' ', [rfReplaceAll]);
  lOneLineTitle := StringReplace(lOneLineTitle, '  ', ' ', [rfReplaceAll]);
  lChart.Canvas.AssignFont(lTitle.Font);

  if lChart.Height < lChart.Canvas.TextWidth(lOneLineTitle) then
    lTitle.Text := WrapText(lOneLineTitle, sLineBreak, [' '], lChart.Height div lTitle.Font.Size)
  else
    lTitle.Text := lOneLineTitle;

  lTitleHeight:=Abs( (lTitle.Text.CountChar(#13) + 1) * lTitle.Font.Height);
  lChart.MarginLeft := Round((lTitleHeight + 5) * 100 / lChart.Width);
end;


constructor TForm1.Create(aOwner: TComponent);
begin
  inherited;
  Series1.FillSampleValues(10);
end;

procedure TForm1.Resize;
var
  lBmp: TBitmap;
  lRect: TRect;
begin
  inherited;
  Image1.Height := ClientHeight div 3;

  lBmp := TBitmap.Create;
  lRect := Rect(0, 0, Image1.Width, Image1.Height);
  try
    AssignChartToGraphic(Chart1, lRect, lBMP, True);
    Image1.Picture.Graphic := lBmp;
  finally
    lBmp.Free;
  end;
end;

end.
